@@ -13,11 +13,20 @@ const char * _sound_service =
 "        <arg type='s' name='player_desktop_name' direction=\"in\"/>\n"
 "        <arg type='b' name='result' direction=\"out\"/>\n"
 "    </method>\n"
-"\n"
 "    <method name = \"GetSoundState\">\n"
 "        <annotation name=\"org.freedesktop.DBus.GLib.Async\" value=\"true\"/>\n"
 "        <arg type='i' name='current_state' direction=\"out\"/>\n"
 "    </method>\n"
+"    <method name = \"EnableTrackSpecificItems\">\n"
+"        <annotation name=\"org.freedesktop.DBus.GLib.Async\" value=\"true\"/>\n"
+"        <arg type='o' name='player_object_path' direction=\"in\"/>\n"
+"        <arg type='s' name='player_desktop_id' direction=\"in\"/>\n"
+"    </method>\n"
+"    <method name = \"EnablePlayerSpecificItems\">\n"
+"        <annotation name=\"org.freedesktop.DBus.GLib.Async\" value=\"true\"/>\n"
+"        <arg type='o' name='player_object_path' direction=\"in\"/>\n"
+"        <arg type='s' name='player_desktop_id' direction=\"in\"/>\n"
+"    </method>    \n"
 "    <signal name=\"SoundStateUpdate\">\n"
 "      <arg name=\"new_state\" type=\"i\" direction=\"out\"/>\n"
 "    </signal>\n"
