@@ -174,6 +174,10 @@ reconnect_to_pulse (gpointer user_data)
 void
 pm_update_volume (gint sink_index, pa_cvolume new_volume)
 {
+  // LP: #850662 
+  if (sink_index < 0){
+    return;
+  }
   pa_operation_unref (pa_context_set_sink_volume_by_index (pulse_context,
                                                            sink_index,
                                                            &new_volume,
@@ -192,6 +196,10 @@ pm_update_mute (gboolean update)
 void
 pm_update_mic_gain (gint source_index, pa_cvolume new_gain)
 {
+  // LP: #850662
+  if (source_index < 0){
+    return;
+  }
   pa_operation_unref (pa_context_set_source_volume_by_index (pulse_context,
                                                              source_index,
                                                              &new_gain,
@@ -202,6 +210,10 @@ pm_update_mic_gain (gint source_index, pa_cvolume new_gain)
 void
 pm_update_mic_mute (gint source_index, gint mute_update)
 {
+    // LP: #850662
+    if (source_index < 0){
+      return;
+    }
     pa_operation_unref (pa_context_set_source_mute_by_index (pulse_context,
                                                              source_index,
                                                              mute_update,
